@@ -43,7 +43,7 @@ static const char *copyright UNUSED =
 /* static char sccsid[] = "@(#)main.c	8.1 (Berkeley) 6/6/93"; */
 /* static char rcsid[] = "$OpenBSD: main.c,v 1.4 1997/01/17 07:13:30 millert Exp $"; */
 static const char *rcsid UNUSED =
- "tftp-hpa $Id: main.c,v 1.8 2001/08/10 18:25:08 hpa Exp $";
+ "tftp-hpa $Id: main.c,v 1.11 2001/11/12 23:27:48 hpa Exp $";
 #endif /* not lint */
 
 /* Many bug fixes are from Jim Guyton <guyton@rand-unix> */
@@ -69,12 +69,12 @@ static const char *rcsid UNUSED =
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
+#ifdef WITH_READLINE
+#include <readline/readline.h>
+#include <readline/history.h>
+#endif
 
 #include "extern.h"
-#include "../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 
 #define	TIMEOUT		5		/* secs between rexmt's */
 #define	LBUFLEN		200		/* size of input buffer */
@@ -86,10 +86,14 @@ int	trace;
 int	verbose;
 int	connected;
 char	mode[32];
+#ifdef WITH_READLINE
+char   *line = NULL;
+#else
 char	line[LBUFLEN];
+#endif
 int	margc;
 char	*margv[20];
-const char *prompt = "tftp";
+const char *prompt = "tftp> ";
 sigjmp_buf	toplevel;
 void	intr(int);
 struct	servent *sp;
@@ -205,6 +209,10 @@ main(int argc, char *argv[])
 	if (sigsetjmp(toplevel,1) != 0)
 		(void)putchar('\n');
 
+#ifdef WITH_READLINE
+	using_history();
+#endif
+
 	command();
 
 	return 0;		/* Never reached */
@@ -349,7 +357,6 @@ put(int argc, char *argv[])
 	}
 	targ = argv[argc - 1];
 	if (strchr(argv[argc - 1], ':')) {
-		char *cp;
 		struct hostent *hp;
 
 		for (n = 1; n < argc - 1; n++)
@@ -386,7 +393,7 @@ put(int argc, char *argv[])
 			printf("putting %s to %s:%s [%s]\n",
 				cp, hostname, targ, mode);
 		peeraddr.sin_port = port;
-		sendfile(fd, targ, mode);
+		tftp_sendfile(fd, targ, mode);
 		return;
 	}
 				/* this assumes the target is a directory */
@@ -404,7 +411,7 @@ put(int argc, char *argv[])
 			printf("putting %s to %s:%s [%s]\n",
 				argv[n], hostname, targ, mode);
 		peeraddr.sin_port = port;
-		sendfile(fd, targ, mode);
+		tftp_sendfile(fd, targ, mode);
 	}
 }
 
@@ -476,7 +483,7 @@ get(int argc, char *argv[])
 				printf("getting from %s:%s to %s [%s]\n",
 					hostname, src, cp, mode);
 			peeraddr.sin_port = port;
-			recvfile(fd, src, mode);
+			tftp_recvfile(fd, src, mode);
 			break;
 		}
 		cp = tail(src);         /* new .. jdg */
@@ -489,7 +496,7 @@ get(int argc, char *argv[])
 			printf("getting from %s:%s to %s [%s]\n",
 				hostname, src, cp, mode);
 		peeraddr.sin_port = port;
-		recvfile(fd, src, mode);
+		tftp_recvfile(fd, src, mode);
 	}
 }
 
@@ -601,7 +608,14 @@ command(void)
 	struct cmd *c;
 
 	for (;;) {
-		printf("%s> ", prompt);
+#ifdef WITH_READLINE
+	        if ( line )
+		     free(line);
+	        line = readline(prompt);
+		if ( !line )
+		     exit(0);	/* EOF */
+#else
+	        fputs(prompt, stdout);
 		if (fgets(line, LBUFLEN, stdin) == 0) {
 			if (feof(stdin)) {
 				exit(0);
@@ -609,11 +623,16 @@ command(void)
 				continue;
 			}
 		}
+#endif
 		if ((line[0] == 0) || (line[0] == '\n'))
 			continue;
+#ifdef WITH_READLINE
+		add_history(line);
+#endif
 		makeargv();
 		if (margc == 0)
 			continue;
+
 		c = getcmd(margv[0]);
 		if (c == (struct cmd *)-1) {
 			printf("?Ambiguous command\n");
