@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpd.c,v 1.30 2001/08/10 18:47:11 hpa Exp $ */
+/* tftp-hpa: $Id: tftpd.c,v 1.33 2001/11/12 10:03:04 hpa Exp $ */
 
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
@@ -35,6 +35,7 @@
  * SUCH DAMAGE.
  */
 
+#include "config.h"		/* Must be included first */
 #include "tftpd.h"
 
 #ifndef lint
@@ -44,7 +45,7 @@ static const char *copyright UNUSED =
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
 /*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftpd.c,v 1.30 2001/08/10 18:47:11 hpa Exp $";
+"tftp-hpa $Id: tftpd.c,v 1.33 2001/11/12 10:03:04 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -70,7 +71,6 @@ static const char *rcsid UNUSED =
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#define __USE_GNU		/* Necessary for basename() on glibc systems */
 #include <string.h>
 #include <stdlib.h>
 #include <pwd.h>
@@ -81,6 +81,10 @@ static const char *rcsid UNUSED =
 #include "recvfrom.h"
 #include "remap.h"
 
+#ifdef HAVE_SYS_FILIO_H
+#include <sys/filio.h>		/* Necessary for FIONBIO on Solaris */
+#endif
+
 #ifdef HAVE_TCPWRAPPERS
 #include <tcpd.h>
 
@@ -89,12 +93,6 @@ int allow_severity	= -1;	/* Don't log at all */
 
 struct request_info wrap_request;
 #endif
-#ifdef HAVE_LIBGEN_H
-#include <libgen.h>		/* Necessary for basename() on Solaris */
-#endif
-#ifdef HAVE_SYS_FILIO_H
-#include <sys/filio.h>		/* Necessary for FIONBIO on Solaris */
-#endif
 
 #define	TIMEOUT 5		/* Default timeout (seconds) */
 #define TRIES   4		/* Number of attempts to send each packet */
@@ -152,7 +150,7 @@ struct options {
   { "blksize",    set_blksize  },
   { "blksize2",   set_blksize2  },
   { "tsize",      set_tsize },
-  { "timeout",	set_timeout  },
+  { "timeout",	  set_timeout  },
   { NULL,         NULL }
 };
 
@@ -204,20 +202,24 @@ main(int argc, char **argv)
   int n;
   int on = 1;
   int fd = 0;
-  int listen = 0;		/* Standalone (listen) mode */
+  int standalone = 0;		/* Standalone (listen) mode */
   char *address = NULL;		/* Address to listen to */
-  int pid;
+  pid_t pid;
   int c;
   int setrv;
-  int timeout = 900;		/* Default timeout */
+  int waittime = 900;		/* Default time to wait for a connect*/
   const char *user = "nobody";	/* Default user */
+  char *p;
 #ifdef WITH_REGEX
   char *rewrite_file = NULL;
 #endif
 
-  __progname = basename(argv[0]);
+  /* basename() is way too much of a pain from a portability standpoint */
+
+  p = strrchr(argv[0], '/');
+  __progname = (p && p[1]) ? p+1 : argv[0];
   
-  openlog(__progname, LOG_PID | LOG_NDELAY, LOG_DAEMON);
+  openlog(__progname, LOG_PID|LOG_NDELAY, LOG_DAEMON);
   
   while ((c = getopt(argc, argv, "csvla:u:r:t:m:")) != -1)
     switch (c) {
@@ -228,13 +230,13 @@ main(int argc, char **argv)
       secure = 1;
       break;
     case 'l':
-      listen = 1;
+      standalone = 1;
       break;
     case 'a':
       address = optarg;
       break;
     case 't':
-      timeout = atoi(optarg);
+      waittime = atoi(optarg);
       break;
     case 'u':
       user = optarg;
@@ -306,7 +308,7 @@ main(int argc, char **argv)
 #endif
 
   /* If we're running standalone, set up the input port */
-  if ( listen ) {
+  if ( standalone ) {
     fd = socket(PF_INET, SOCK_DGRAM, 0);
     
     memset(&bindaddr, 0, sizeof bindaddr);
@@ -357,7 +359,7 @@ main(int argc, char **argv)
 
     /* Daemonize this process */
     {
-      int f = fork();
+      pid_t f = fork();
       if ( f > 0 )
 	exit(0);
       if ( f < 0 ) {
@@ -375,8 +377,12 @@ main(int argc, char **argv)
   }
 
   /* This means we don't want to wait() for children */
+#ifdef SA_NOCLDWAIT
+  set_signal(SIGCHLD, SIG_IGN, SA_NOCLDSTOP|SA_NOCLDWAIT);
+#else
   set_signal(SIGCHLD, SIG_IGN, SA_NOCLDSTOP);
-  
+#endif
+
   /* Take SIGHUP and use it to set a variable.  This
      is polled synchronously to make sure we don't
      lose packets as a result. */
@@ -384,12 +390,12 @@ main(int argc, char **argv)
   
   while ( 1 ) {
     fd_set readset;
-    struct timeval tv_timeout;
+    struct timeval tv_waittime;
     int rv;
     
     if ( caught_sighup ) {
       caught_sighup = 0;
-      if ( listen ) {
+      if ( standalone ) {
 #ifdef HAVE_REGEX
 	if ( rewrite_file ) {
 	  freerules(rewrite_rules);
@@ -404,11 +410,11 @@ main(int argc, char **argv)
     
     FD_ZERO(&readset);
     FD_SET(fd, &readset);
-    tv_timeout.tv_sec = timeout;
-    tv_timeout.tv_usec = 0;
+    tv_waittime.tv_sec = waittime;
+    tv_waittime.tv_usec = 0;
     
-    /* Never time out if we're in listen mode */
-    rv = select(fd+1, &readset, NULL, NULL, listen ? NULL : &tv_timeout);
+    /* Never time out if we're in standalone mode */
+    rv = select(fd+1, &readset, NULL, NULL, standalone ? NULL : &tv_waittime);
     if ( rv == -1 && errno == EINTR )
       continue;		/* Signal caught, reloop */
     if ( rv == -1 ) {
@@ -423,7 +429,7 @@ main(int argc, char **argv)
 		   (struct sockaddr *)&from, &fromlen,
 		   &myaddr);
 
-    if ( listen && myaddr.sin_addr.s_addr == INADDR_ANY ) {
+    if ( standalone && myaddr.sin_addr.s_addr == INADDR_ANY ) {
       /* myrecvfrom() didn't capture the source address; but we might
 	 have bound to a specific address, if so we should use it */
       memcpy(&myaddr.sin_addr, &bindaddr.sin_addr, sizeof bindaddr.sin_addr);
@@ -534,8 +540,8 @@ main(int argc, char **argv)
 
 char   *rewrite_access(char *, int);
 int	validate_access(char *, int, struct formats *);
-void	sendfile(struct formats *, struct tftphdr *, int);
-void	recvfile(struct formats *, struct tftphdr *, int);
+void	tftp_sendfile(struct formats *, struct tftphdr *, int);
+void	tftp_recvfile(struct formats *, struct tftphdr *, int);
 
 struct formats {
   const char *f_mode;
@@ -545,8 +551,8 @@ struct formats {
   void	(*f_recv)(struct formats *, struct tftphdr *, int);
   int	f_convert;
 } formats[] = {
-  { "netascii",   rewrite_access, validate_access, sendfile, recvfile, 1 },
-  { "octet",	rewrite_access, validate_access, sendfile, recvfile, 0 },
+  { "netascii",   rewrite_access, validate_access, tftp_sendfile, tftp_recvfile, 1 },
+  { "octet",	rewrite_access, validate_access, tftp_sendfile, tftp_recvfile, 0 },
   { NULL, NULL, NULL, NULL, NULL, 0 }
 };
 
@@ -650,9 +656,12 @@ int
 set_blksize(char *val, char **ret)
 {
   static char b_ret[6];
-  unsigned int sz = atoi(val);
+  unsigned int sz;
+  char *vend;
   
-  if ( blksize_set )
+  sz = (unsigned int)strtoul(val, &vend, 10);
+  
+  if ( blksize_set || *vend )
     return 0;
   
   if (sz < 8)
@@ -675,9 +684,12 @@ int
 set_blksize2(char *val, char **ret)
 {
   static char b_ret[6];
-  unsigned int sz = atoi(val);
+  unsigned int sz;
+  char *vend;
   
-  if ( blksize_set )
+  sz = (unsigned int)strtoul(val, &vend, 10);
+  
+  if ( blksize_set || *vend )
     return 0;
   
   if (sz < 8)
@@ -710,15 +722,19 @@ set_blksize2(char *val, char **ret)
 int
 set_tsize(char *val, char **ret)
 {
-  static char b_ret[sizeof(off_t)*CHAR_BIT/3+2];
-  off_t sz = atol(val);
+  static char b_ret[sizeof(uintmax_t)*CHAR_BIT/3+2];
+  uintmax_t sz;
+  char *vend;
+
+  sz = strtoumax(val, &vend, 10);
   
-  if ( !tsize_ok )
+  if ( !tsize_ok || *vend )
     return 0;
   
   if (sz == 0)
-    sz = tsize;
-  sprintf(*ret = b_ret, "%lu", sz);
+    sz = (uintmax_t)tsize;
+
+  sprintf(*ret = b_ret, "%"PRIuMAX, sz);
   return(1);
 }
 
@@ -729,9 +745,12 @@ int
 set_timeout(char *val, char **ret)
 {
   static char b_ret[4];
-  unsigned long to = atol(val);
-  
-  if ( to < 1 || to > 255 )
+  unsigned long to;
+  char *vend;
+
+  to = strtoul(val, &vend, 10);
+
+  if ( to < 1 || to > 255 || *vend )
     return 0;
   
   timeout    = to;
@@ -903,7 +922,7 @@ timer(int sig)
  * Send the requested file.
  */
 void
-sendfile(struct formats *pf, struct tftphdr *oap, int oacklen)
+tftp_sendfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 {
   struct tftphdr *dp;
   struct tftphdr *ap;		/* ack packet */
@@ -1011,7 +1030,7 @@ justquit(int sig)
  * Receive a file.
  */
 void
-recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
+tftp_recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 {
   struct tftphdr *dp;
   int n, size;
